/*****************************************************************************/
/* N.A.D.A.R. - A network tank battle game.                                  */
/*                                                                           */
/* N.A.D.A.R. Copyright (c) 2000 Sakai Hiroaki.                              */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* N.A.D.A.R. is a network tank battle game. You get into a tank,            */
/* operate it, and destroy other tanks to survive!                           */
/*****************************************************************************/

/*****************************************************************************/
/* 饤ȤΥᥤ                                                      */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include "Client.h"
#include "etc.h"

Client client = NULL;

void quit (int value)
{
  char buffer[50];

  /*
  fprintf(stderr, "Terminated.\n");
  */

  if (client) {
    PathSendCharacters(PlayerGetPath(ClientGetPlayer(client)), DELETE_ME, 0);
    sleep(3);
  }

  exit (0);
}

static char * player_names[] = {
  "Bolken", "WildDog", "Hornet", "Airi", "Akana",
  "GreenTea", "Dindon", "Ripple", "RingOnTheWater", "Blaky",
  "HalfPot", "SmartBomb", "Flogger", "DirtyHorse", "HungryStomach",
  "SlamPoint", "RattleSnake", "GrindBoy", "SqueakVoice", "JaggerEdge",
  "Belch", "WoodenDoll", "NimbleBoy", "PlayingHookey", "DrippingWet",
  "SillyBoy", "VeryBottom", "RapidlyLike", "Nawo", "ScrewDriver",
  "NezumiKozou", "Nababa", "Yuuki", "NipponoFace", "WhoAmI?",
  "FuurinKazan", "Polpfun", "LZ", "I.L.J.", "Folcman",
  "Goemon", "Jorjor", "BerkenIII", "Kame", "Sushi",
  "JarringSound", "Jumble", "MixJuice", "CupCake", "CottageCheese",
  "Gingin", "Kennel", "CoffeeBean", "JunkFood", "HighwayPatrol"
};

#define PLAYER_NAMES_NUMBER ((int)(sizeof(player_names)/sizeof(player_names[0])))

int main(int argc, char * argv[])
{
  char * server_hostname;
  int port;
  char * player_name;
  int size;

  signal(SIGINT , quit);
  signal(SIGTERM, quit);

  InitializeRandom();

  server_hostname = GetArgument(&argc, argv, "-server");
  if (server_hostname == NULL)
    server_hostname = GetArgument(&argc, argv, "-s");

  if (server_hostname == NULL) {
    printf("\n"
	   "                                 N.A.D.A.R.\n"
	   "                        A network tank battle game.\n"
	   "\n"
	   "               N.A.D.A.R. Copyright (c) 2000 Sakai Hiroaki.\n"
	   "                            All Rights Reserved.\n"
	   "\n"
	   "N.A.D.A.R. is a client-server model program. "
	   "This is N.A.D.A.R.\'s client.\n"
	   "\n"
	   "You have to start server program before playing.\n"
	   "Start server program on a server machine as\n"
	   "\n"
	   "  %% nadars\n"
	   "\n"
	   "And start client program as\n"
	   "\n"
	   "  %% nadar -s [Server's hostname] -p [Player\'s name]\n"
	   "\n"
	   "If playing speed is very slow, "
	   "run clients as below with size option.\n"
	   "\n"
	   "  %% nadar -s [Server's hostname] -p [Player\'s name] -size 30\n"
	   "\n");
    exit (0);
  }

  port = GetArgumentByIntegerValue(&argc, argv, "-port");
  if (port == -1) port = PORT;

  player_name = GetArgument(&argc, argv, "-player");
  if (player_name == NULL)
    player_name = GetArgument(&argc, argv, "-p");
  if (player_name == NULL)
    player_name = player_names[GetRandomValue(PLAYER_NAMES_NUMBER)];

  size = GetArgumentByIntegerValue(&argc, argv, "-size");
  if (size < 3 || size > 500) size = DISPLAY_DEFAULT_SIZE;

  client = ClientCreate(server_hostname, port, player_name, size, &argc, argv);
  if (client == NULL) ErrorExit("Cannot create client.");

  ClientStart(client);

  client = ClientDestroy(client);

  exit (0);
}

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
