/*****************************************************************************/
/* N.A.D.A.R. - A network tank battle game.                                  */
/*                                                                           */
/* N.A.D.A.R. Copyright (c) 2000 Sakai Hiroaki.                              */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* N.A.D.A.R. is a network tank battle game. You get into a tank,            */
/* operate it, and destroy other tanks to survive!                           */
/*****************************************************************************/

/*****************************************************************************/
/* Stage 饹(ѤΥ饹)Υ饤֥                        */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                   */
/*****************************************************************************/

#include "StageP.h"

/*****************************************************************************/
/* ؿ                                                                */
/*****************************************************************************/

/*===========================================================================*/
/* ѿμ                                                    */
/*===========================================================================*/

Map StageGetMap(Stage s) {return (s->map);}
ObjList StageGetTankList(     Stage s) {return (s->tank_list     );}
ObjList StageGetCanonList(    Stage s) {return (s->canon_list    );}
ObjList StageGetMissileList(  Stage s) {return (s->missile_list  );}
ObjList StageGetExplosionList(Stage s) {return (s->explosion_list);}

/*===========================================================================*/
/* ֥ȤȺ                                                  */
/*===========================================================================*/

/*---------------------------------------------------------------------------*/
/* ֥Ȥ                                                        */
/*---------------------------------------------------------------------------*/

int StageCreateTankForPlayer(Stage stage, Player player)
{
  Tank tank;
  int x, y, tx, ty, f;
  ObjListData current;
  double rx, ry, dx, dy, r;
  Tank other;

  r = ((double)TANK_SIZE / CHARACTER_SIZE) * 2.0;

  /* ᤯̵̤֤ɤå */
  do {
    rx = GetRandomValue(MAP_WIDTH ) + 0.5;
    ry = GetRandomValue(MAP_HEIGHT) + 0.5;
    f = 0;
    for (current = ObjList_GetStart(stage->tank_list);
	 !ObjList_IsEndEdge(stage->tank_list, current);
	 current = ObjListData_GetNext(current)) {
      other = (Tank)ObjListData_GetObj(current);
      dx = TankGetX(other) - rx;
      dy = TankGetY(other) - ry;
      if (dx * dx + dy * dy <= r * r) {
	f = 1;
	break;
      }
    }
  } while (f);

  tank = TankCreate(rx, ry, GetRandomValue(TANK_ANGLE_NUMBER),
		    player, TANK_ACTIVE);
  if (tank == NULL) ErrorExit("Cannot create tank.");
  PlayerSetTank(player, tank);
  ObjList_InsertObjToEnd(stage->tank_list, tank, (ObjDestructor)TankDestroy);

  /* ֤μ򥯥ꥢ */
  for (x = -1; x <= 1; x++) {
    for (y = -1; y <= 1; y++) {
      tx = ConvertXFromPositionToMap(TankGetX(tank)) + x;
      ty = ConvertYFromPositionToMap(TankGetY(tank)) + y;
      if (MapIsInside(stage->map, tx, ty))
	MapChangeState(stage->map, tx, ty, 0);
    }
  }
  return (0);
}

Stage StageCreate(int stage_number, ObjList player_list)
{
  Stage stage;
  Player player;
  Tank tank;
  int i, x, y, tx, ty;
  ObjListData current;

  stage = (Stage)malloc(sizeof(_Stage));
  if (stage == NULL) return (NULL);

  stage->map = MapCreate(stage, MAP_WIDTH, MAP_HEIGHT, stage_number);
  if (stage->map == NULL) ErrorExit("Cannot create stage->map.");

  stage->stage_number = stage_number;
  stage->player_list = player_list;
  stage->tank_list      = ObjList_Create();
  stage->canon_list     = ObjList_Create();
  stage->missile_list   = ObjList_Create();
  stage->explosion_list = ObjList_Create();

  for (current = ObjList_GetStart(player_list);
       !ObjList_IsEndEdge(player_list, current);
       current = ObjListData_GetNext(current)) {
    player = (Player)ObjListData_GetObj(current);
    StageCreateTankForPlayer(stage, player);
  }

  return (stage);
}

/*---------------------------------------------------------------------------*/
/* ֥Ȥκ                                                        */
/*---------------------------------------------------------------------------*/

Stage StageDestroy(Stage stage)
{
  ObjListData current;
  Player player;

  if (stage == NULL) return (NULL);

  if (stage->map) MapDestroy(stage->map);

  for (current = ObjList_GetStart(stage->player_list);
       !ObjList_IsEndEdge(stage->player_list, current);
       current = ObjListData_GetNext(current)) {
    player = (Player)ObjListData_GetObj(current);
    PlayerSetTank(player, NULL);
  }
  if (stage->tank_list)      ObjList_Destroy(stage->tank_list);
  if (stage->canon_list)     ObjList_Destroy(stage->canon_list);
  if (stage->missile_list)   ObjList_Destroy(stage->missile_list);
  if (stage->explosion_list) ObjList_Destroy(stage->explosion_list);

  free(stage);

  return (NULL);
}

/*---------------------------------------------------------------------------*/
/* Stage is over?                                                            */
/*---------------------------------------------------------------------------*/

static int StageIsOver(Stage stage, int lose)
{
  switch (ObjList_GetLength(stage->player_list)) {
  case 1 :
    return (lose >= 1);
  case 2 :
    return (lose >= 1);
  case 3 :
    return (lose >= 2);
  case 4 :
    return (lose >= 2);
  case 5 :
    return (lose >= 3);
  default :
    return (lose >= ObjList_GetLength(stage->player_list) * 2 / 3);
  }
  return (0);
}

/*---------------------------------------------------------------------------*/
/* ѤΥǡκ                                                      */
/*---------------------------------------------------------------------------*/

static char * StageMakeData(Stage stage, char * p)
{
  Tank tank;
  Canon canon;
  Missile missile;
  Explosion explosion;
  ObjListData current;

  p = MapMakeData(stage->map, p);

  for (current = ObjList_GetStart(stage->tank_list);
       !ObjList_IsEndEdge(stage->tank_list, current);
       current = ObjListData_GetNext(current)) {
    tank = (Tank)ObjListData_GetObj(current);
    p = TankMakeDrawData(tank, p);
  }

  for (current = ObjList_GetStart(stage->canon_list);
       !ObjList_IsEndEdge(stage->canon_list, current);
       current = ObjListData_GetNext(current)) {
    canon = (Canon)ObjListData_GetObj(current);
    p = CanonMakeDrawData(canon, p);
  }

  for (current = ObjList_GetStart(stage->missile_list);
       !ObjList_IsEndEdge(stage->missile_list, current);
       current = ObjListData_GetNext(current)) {
    missile = (Missile)ObjListData_GetObj(current);
    p = MissileMakeDrawData(missile, p);
  }

  for (current = ObjList_GetStart(stage->explosion_list);
       !ObjList_IsEndEdge(stage->explosion_list, current);
       current = ObjListData_GetNext(current)) {
    explosion = (Explosion)ObjListData_GetObj(current);
    p = ExplosionMakeDrawData(explosion, p);
  }

  *p = '\0';

  return (p);
}

/*---------------------------------------------------------------------------*/
/* ƥȤɽ                                                            */
/*---------------------------------------------------------------------------*/

static int StageSendTextData(Stage stage)
{
  char buffer[1000];
  char * p;
  Player player;
  ObjListData current;

  PlayerSendMessageToAllPlayers(stage->player_list, TEXT_CLEAR "\t\n", 0);

  p = buffer;

  sprintf(p,
	  TEXT_DRAW "\t0      N.A.D.A.R.\n"
	  TEXT_DRAW "\t0  Copyright (c) 2000\n"
	  TEXT_DRAW "\t0    Sakai Hiroaki.\n"
	  TEXT_DRAW "\t0 \n");
  p += strlen(p);

  sprintf(p, TEXT_DRAW "\t%d   Stage : %d\n" TEXT_DRAW "\t0 \n",
	  0, stage->stage_number + 1);
  p += strlen(p);

  for (current = ObjList_GetStart(stage->player_list);
       !ObjList_IsEndEdge(stage->player_list, current);
       current = ObjListData_GetNext(current)) {
    player = (Player)ObjListData_GetObj(current);
    sprintf(p, TEXT_DRAW "\t%d   %s\n",
	    PlayerGetColorNumber(player), PlayerGetName(player));
    p += strlen(p);
    sprintf(p, TEXT_DRAW "\t%d   Win:%d Lose:%d\n",
	    PlayerGetColorNumber(player),
	    PlayerGetWin(player), PlayerGetLose(player));
    p += strlen(p);
    sprintf(p, TEXT_DRAW "\t0 \n");
    p += strlen(p);
  }
  PlayerSendMessageToAllPlayers(stage->player_list, buffer, 0);

  return (0);
}

/*===========================================================================*/
/* ơγ(ФΥᥤ롼)                                      */
/*===========================================================================*/

char * StageAllocateBuffer(Stage stage, char * buffer)
{
  int size;

  size = ObjList_GetLength(stage->player_list) * (16 * 14);
  if (buffer != NULL) free(buffer);
  buffer = (char *)malloc(sizeof(char) * size);
  if (buffer == NULL) ErrorExit("Cannot allocate memory for buffer.");

  return (buffer);
}

/*===========================================================================*/
/* ǡž̤ưȡ®٤٤ʤΤǡ                          */
/* ѥåɤƥǡ̤Ĵ                                          */
/*===========================================================================*/

static int StagePuddingBuffer(Stage stage, char * buffer)
{
  int len;

  len = strlen(buffer);

#if 0
  while (len < ObjList_GetLength(stage->player_list) * (16 * 8) - 30) {
    strcat(buffer + len, DUMMY "\tDDDDDDDDDDDDDDDDDDDD\n");
    len += strlen(buffer + len);
  }
  if (len < ObjList_GetLength(stage->player_list) * (16 * 8) - 10) {
    strcat(buffer + len, DUMMY "\t");
    len += strlen(buffer + len);
    while (len < ObjList_GetLength(stage->player_list) * (16 * 8))
      buffer[len++] = 'D';
    buffer[len++] = '\n';
    buffer[len  ] = '\0';
  }
#endif

  return (len);
}

int StageStartAsServer(Stage stage, ObjList player_list, Path request)
{
  char * buffer; /* ǡѥХåե */
  char * p;
  char * tmp;
  char command[100];
  char player_name[100];
  int len;
  int state;
  Path path;
  Player player;
  Tank tank;
  Canon canon;
  Missile missile;
  Explosion explosion;
  int lose = 0;
  int f0, f1;
  ObjListData current;

  /* ѥХåեγ */
  buffer = StageAllocateBuffer(stage, NULL);

  /* ǥץ쥤Υꥢ */
  PlayerSendMessageToAllPlayers(stage->player_list, DISPLAY_CLEAR "\t\n", 0);

  /* ɽƥȥǡ */
  StageSendTextData(stage);

  /* ޥåץǡ */
  MapSendChangesToAllPlayers(stage->map, stage->player_list);

  PlayerSendMessageToAllPlayers(stage->player_list,
				READ_KEY "\t\n" SEND_KEY "\t\n", 0);

  while (!StageIsOver(stage, lose)) { /* ơΥᥤ롼 */

    /* 饤Ȥμդ */
    PathCheckRecvState(request);
    if (PathIsRecvStateActive(request)) {
      path = PathCreateToCommunicateWithClient(request);
      PathRecvCharacters(path, buffer, 100);
      sscanf(buffer, "%s %s", command, player_name);

      if (!strcmp(command, JOIN_REQUEST)) {
        player = PlayerCreate(player_name, path, PLAYER_ACTIVE);
        ObjList_InsertObjToEnd(player_list, player,
			       (ObjDestructor)PlayerDestroy);
#if 0
        PlayerMakeData(player, buffer);
        PathSendCharacters(PlayerGetPath(player), buffer, 0);
#endif

	PathSendCharacters(PlayerGetPath(player), DISPLAY_CLEAR "\t\n", 0);

	MapSendAllMap(stage->map, player);

	StageCreateTankForPlayer(stage, player);

	/* ѥХåեγ */
	buffer = StageAllocateBuffer(stage, buffer);

	/* ɽƥȥǡ */
	StageSendTextData(stage);

	PathSendCharacters(PlayerGetPath(player),
			   READ_KEY "\t\n" SEND_KEY "\t\n", 0);

      } else {
        path = PathDestroy(path);
      }
    }

    /* ȯν */
    for (current = ObjList_GetStart(stage->explosion_list);
	 !ObjList_IsEndEdge(stage->explosion_list, current);
	 current = ObjListData_GetNext(current)) {
      explosion = (Explosion)ObjListData_GetObj(current);
      state = ExplosionAct(explosion, stage->map);
      switch (state) {
      case EXPLOSION_DESTROY :
	current = ObjList_DeleteObjToPrev(stage->explosion_list, current);
	break;
      default :
	break;
      }
    }

    /* ˤƤν */
    for (current = ObjList_GetStart(stage->canon_list);
	 !ObjList_IsEndEdge(stage->canon_list, current);
	 current = ObjListData_GetNext(current)) {
      canon = (Canon)ObjListData_GetObj(current);
      state = CanonAct(canon, stage);
      switch (state) {
      case CANON_EXPLOSION :
	explosion = ExplosionCreate(CanonGetX(canon), CanonGetY(canon),
				    CanonGetTank(canon), EXPLOSION_ACTIVE);
	ObjList_InsertObjToEnd(stage->explosion_list, explosion,
			       (ObjDestructor)ExplosionDestroy);
	CanonSetState(canon, CANON_DESTROY);
	/* Fall through to destroy the object */
      case CANON_DESTROY :
	current = ObjList_DeleteObjToPrev(stage->canon_list, current);
	break;
      default :
	break;
      }
    }

    /* ߥν */
    for (current = ObjList_GetStart(stage->missile_list);
	 !ObjList_IsEndEdge(stage->missile_list, current);
	 current = ObjListData_GetNext(current)) {
      missile = (Missile)ObjListData_GetObj(current);
      state = MissileAct(missile, stage);
      switch (state) {
      case MISSILE_EXPLOSION :
	explosion = ExplosionCreate(MissileGetX(missile),
				    MissileGetY(missile),
				    MissileGetTank(missile),
				    EXPLOSION_ACTIVE);
	ObjList_InsertObjToEnd(stage->explosion_list, explosion,
			       (ObjDestructor)ExplosionDestroy);
	MissileSetState(missile, MISSILE_DESTROY);
	/* Fall through to destroy the object */
      case MISSILE_DESTROY :
	current = ObjList_DeleteObjToPrev(stage->missile_list, current);
	break;
      default :
	break;
      }
    }

    /*  Player ǡȤ */
    f1 = 1;
    do {
      f0 = 0;
      if (!f1) usleep(1000); /* Wait in 1 millisecond. */
      f1 = 0;

      for (current = ObjList_GetStart(stage->player_list);
	   !ObjList_IsEndEdge(stage->player_list, current);
	   current = ObjListData_GetNext(current)) {
	player = (Player)ObjListData_GetObj(current);
	if (PlayerGetState(player) != PLAYER_ACTIVE) continue;
	f0 = 1;
	PathCheckRecvState(PlayerGetPath(player));
	if (!PathIsRecvStateActive(PlayerGetPath(player))) continue;
	f1 = 1;
	PathRecvCharacters(PlayerGetPath(player), command, 20);
	if (!strcmp(command, DELETE_ME)) {
	  PathSendCharacters(PlayerGetPath(player), DELETE_YOU, 0);
	  TankSetState(PlayerGetTank(player), TANK_EXPLOSION);
	  PlayerSetState(player, PLAYER_DESTROY);
	} else {
	  TankSetAction(PlayerGetTank(player), strtoul(command, &tmp, 10));
	  PlayerSetState(player, PLAYER_DONE);
	}
      }
    } while (f0);

    for (current = ObjList_GetStart(stage->player_list);
	 !ObjList_IsEndEdge(stage->player_list, current);
	 current = ObjListData_GetNext(current)) {
      player = (Player)ObjListData_GetObj(current);
      if (PlayerGetState(player) == PLAYER_DONE) 
	PlayerSetState(player, PLAYER_ACTIVE);
    }

    /* ֤ν */
    for (current = ObjList_GetStart(stage->tank_list);
	 !ObjList_IsEndEdge(stage->tank_list, current);
	 current = ObjListData_GetNext(current)) {
      tank = (Tank)ObjListData_GetObj(current);
      state = TankAct(tank, stage);
      switch (state) {
      case TANK_EXPLOSION :
	explosion = ExplosionCreate(TankGetX(tank), TankGetY(tank),
				    tank, EXPLOSION_ACTIVE);
	ObjList_InsertObjToEnd(stage->explosion_list, explosion,
			       (ObjDestructor)ExplosionDestroy);
	TankSetState(tank, TANK_EXPLOSING);
	TankSetDestroyTimer(tank, TANK_DESTROY_INTERVAL);
	break;
      case TANK_EXPLOSING :
	TankDecDestroyTimer(tank);
	if (TankGetDestroyTimer(tank) <= 0) {
	  TankSetState(tank, TANK_DESTROY);
	  lose++;
	}
	break;
      case TANK_DESTROY :
	/* current = ObjList_DeleteObjToPrev(stage->tank_list, current); */
	break;
      default :
	break;
      }
    }

    /* ѥǡκ */
    /* ˡǡ٤ʬȤʤ٤ʤΤǡ
       ҤȤĤˤޤȤƤääƤ */
    p = StageMakeData(stage, buffer);

#ifdef PEEPING
    for (current = ObjList_GetStart(stage->player_list);
	 !ObjList_IsEndEdge(stage->player_list, current);
	 current = ObjListData_GetNext(current)) {
      player = (Player)ObjListData_GetObj(current);
      if (PlayerGetState(player) != PLAYER_ACTIVE) continue;
      sprintf(p, DISPLAY_COPY "\t%3.1f %3.1f\n"
	      GAME_CLEAN "\t\n"
	      READ_KEY "\t\n"
	      SEND_KEY "\t\n",
	      TankGetX(PlayerGetTank(player)),
	      TankGetY(PlayerGetTank(player)));

      len = StagePuddingBuffer(stage, buffer);

      PathSendCharacters(PlayerGetPath(player), buffer, len);
    }
#else
    sprintf(p, DISPLAY_COPY "\t0.0 0.0\n"
	    GAME_CLEAN "\t\n"
	    READ_KEY "\t\n"
	    SEND_KEY "\t\n");

    len = StagePuddingBuffer(stage, buffer);

    for (current = ObjList_GetStart(stage->player_list);
	 !ObjList_IsEndEdge(stage->player_list, current);
	 current = ObjListData_GetNext(current)) {
      player = (Player)ObjListData_GetObj(current);
      if (PlayerGetState(player) != PLAYER_ACTIVE) continue;
      PathSendCharacters(PlayerGetPath(player), buffer, len);
    }
#endif

  }
  free(buffer);

  return (0);
}

/*****************************************************************************/
/* ޤ                                                                  */
/*****************************************************************************/

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
